///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging; 
using System.Data;
using System.Windows.Forms;
using ILNumerics.Exceptions; 
using System.Resources; 
using OpenTK.Graphics; 
using OpenTK.Graphics.OpenGL;
using OpenTK.Platform;
using OpenTK.Graphics.OpenGL.Enums;
using ILNumerics.Drawing.Graphs; 
using ILNumerics.Drawing; 
using ILNumerics.Drawing.Labeling; 
using ILNumerics.Drawing.Interfaces; 
using ILNumerics.Drawing.Shapes; 

namespace ILNumerics.Drawing.Platform.OpenGL
{
    /// <summary>
    /// OpenGL implementation for ILPlot2DGraph
    /// </summary>
    public class ILOGLPlot2DGraph : ILPlot2DGraph {

        #region attributes 

        protected int m_textureName = 0; 
        protected float[] m_markerQuads; 

        #endregion

        #region constructors 

        internal ILOGLPlot2DGraph(ILPanel panel, ILArray<float> sourceArray,
                                  ILClippingData clippingContainer) 
                                : base(panel, sourceArray,clippingContainer) {
        }
        internal ILOGLPlot2DGraph(ILPanel panel, ILArray<float> xData, ILArray<float> yData,
                                  ILClippingData clippingContainer) 
                                : base(panel, xData, yData, clippingContainer) {
        }

        #endregion

        #region public (& abstract) interface 

        public override void Draw(ILRenderProperties p) {
            //System.Diagnostics.Debug.WriteLine("ILOGLGraphPlot2D_Draw"); 
            #region draw line
            // draw linestrip - simple case - fast vertex rendering 
            if (m_properties.Visible) {
                GL.Disable(EnableCap.Blend); 
                GL.Disable(EnableCap.Texture2D);
                ILOGLPanel.SetupLineStyle(m_properties); 
                unsafe { 
                    fixed(C4bV3f* pVertArr = m_vertices) {  
                        GL.InterleavedArrays(InterleavedArrayFormat.C4ubV3f,0,(IntPtr)pVertArr); 
                        if (m_properties.Color.IsEmpty) {
                            GL.ShadeModel(ShadingModel.Smooth); 
                        } else {
                            GL.ShadeModel(ShadingModel.Flat); 
                            GL.DisableClientState(EnableCap.ColorArray); 
                        }

                        if (m_startID < 2) {
                            GL.DrawArrays(BeginMode.LineStrip, m_startID, m_vertexCount-1);
                        } else {
                            GL.DrawArrays(BeginMode.LineStrip, m_startID, m_vertexCount-m_startID);
                            GL.DrawArrays(BeginMode.LineStrip, 0, m_startID);
                        }
                    }
                }
            }
            #endregion
            #region draw markers
            if (m_marker.Visible) { 
                m_marker.Shape.Draw(p, m_marker, m_vertices,0, m_vertexCount-1);
            }
            #endregion
        }
        public override void Dispose() {
            m_vertices = null;
        }
        /// <summary>
        /// (re)create vertex buffer, line-, texture objects
        /// </summary>
        internal override void Configure() {
        }

        public override void DrawToLegend(ILRenderProperties p, Rectangle sampleArea, Rectangle labelArea) {
            if (p.Graphics != null) {
                // bitmap output is handled by base class
                base.DrawToLegend(p, sampleArea, labelArea);
                return; 
            }
            int sampY = (int)(sampleArea.Top + sampleArea.Height / 2.0f);
            #region draw line
            if (m_properties.Visible) {
                ILNumerics.Drawing.Platform.OpenGL.ILOGLPanel.SetupLineStyle(m_properties); 
                GL.Begin(BeginMode.Lines); 
                GL.Vertex2(sampleArea.Left,sampY); 
                GL.Vertex2(sampleArea.Right,sampY); 
                GL.End();
            }
            #endregion

            #region draw marker 
            if (m_marker.Visible) {
                C4bV3f pos = new C4bV3f();
                pos.XPosition = sampleArea.Left + sampleArea.Width / 2.0f; 
                pos.YPosition = sampY; 
                m_marker.Shape.Draw(p, m_marker,new C4bV3f[] {pos},0,-1);
            }
            #endregion
            //GL.Vertex2(sampleArea.Right,sampleArea.Bottom); 
            //GL.Vertex2(sampleArea.Left,sampleArea.Bottom); 
            m_label.m_position.X = labelArea.X + labelArea.Width / 2;
            m_label.m_position.Y = labelArea.Y + labelArea.Height / 2;
            m_label.Anchor = new PointF(.5f, .5f);  //  = TickLabelAlign.center | TickLabelAlign.vertCenter; 
            m_label.Draw(p); 
        }
        #endregion
    }
}
